<?php
/* --------------------------------------------------------------
   Amazon Pay  V3.1.0
   AlkimAmazonLoginAndPayHandler.class.php 2017-02-10

   alkim media
   http://www.alkim.de

   patworx multimedia GmbH
   http://www.patworx.de/

   Released under the GNU General Public License
   --------------------------------------------------------------
*/
class AlkimAmazonLoginAndPayHandler
{
    public static $client;
    public static function getConfig()
    {
        $config = array();
        $config['sandbox'] = (MODULE_PAYMENT_AM_APA_MODE != 'live');
        $config['merchant_id'] = AWS_MERCHANT_ID;
        $config['access_key'] = AWS_ACCESS_KEY_ID;
        $config['secret_key'] = AWS_SECRET_ACCESS_KEY;
        $config['client_id'] = MODULE_PAYMENT_AM_APA_CLIENTID;
        $config['application_name'] = 'xtcm-am_apa-3';
        $config['application_version'] = AMZ_FILE_VERSION;
        $config['region'] = 'de';
        return $config;
    }


    public static function getClient()
    {
        if (!isset(self::$client) || !is_object(self::$client)) {
            self::$client = new \PayWithAmazon\Client(self::getConfig());
        }
        return self::$client;
    }


    public static function throwException($msg)
    {
        throw new Exception($msg);
    }

    public static function call($action, $parameters, $waitingTimeOnThrottle = 1)
    {
        $cacheId = 'amz_'.$action.'_'.md5(serialize($parameters));
        $cacheFile = DIR_FS_CATALOG.'cache/'.$cacheId.'.cache';
        $timeFile = DIR_FS_CATALOG.'cache/'.$cacheId.'.cachetime';
        if (file_exists($cacheFile) && (float)file_get_contents($timeFile) > microtime(true)-0.5) {
            return unserialize(file_get_contents($cacheFile));
        } else {
            $client = self::getClient();
            $callFailed = false;
            try{
            $return = $client->{$action}($parameters)->toArray();
            }catch(Exception $e){
                $callFailed = true;
                $return = array(
                    'Error'=>array(
                        'Code'=>'ProbablyThrottled',
                        'Message'=>str_replace(array(PHP_EOL, "\n"), array(" ", " "), $e->getMessage())
                    )
                );
            }
            if ($return["ResponseStatus"] != '200') {
                if ($return["Error"]) {
                    AlkimAmazonHandler::log('exception', array('orderRef'=>$parameters['amazon_order_reference_id'], 'txnId'=> '', 'summary'=>$action.' exception: '.$return["Error"]["Message"]), print_r($requestParameters, true), print_r($return, true));
                    if($return["Error"]["Code"] == 'RequestThrottled' || $return["Error"]["Code"] == 'ProbablyThrottled'){
                        if($watingTimeOnThrottle <= 4){
                            sleep(pow(2,$waitingTimeOnThrottle));
                            return self::call($action, $parameters, $waitingTimeOnThrottle+1);
                        }
                    }
                } else {
                    AlkimAmazonHandler::log('exception', array('orderRef'=>$parameters['amazon_order_reference_id'], 'txnId'=> '', 'summary'=>$action.' exception'), print_r($requestParameters, true), print_r($return, true));
                }
                /*echo '<pre>';
                echo 'Something is Wrong:'."\n";
                var_dump($return);
                die;*/
            }
            file_put_contents($cacheFile, serialize($return));
            file_put_contents($timeFile, microtime(true));
            return $return;
        }
    }

    public static function getUserInfo($accessToken)
    {
        $client = self::getClient();
        return $client->getUserInfo($accessToken);
    }
}
